#from typing_extensions import Required
from rest_framework import serializers
from .models import * 

# ############## User%Module #################



class SessionSerializer(serializers.ModelSerializer):
    class Meta:
        model = Session
        fields="__all__"

class SessionMiniSerializer(serializers.ModelSerializer):
    class Meta:
        model = Session
        fields=("id",'name')

class PatientSessionItemMiniSerializer(serializers.ModelSerializer):
    pt_fullname = serializers.CharField(source='session.patient.fullname',read_only=True)
    patient_id = serializers.IntegerField(source='session.patient.id',read_only=True)
    s_count = serializers.IntegerField(source='session.sessions_count',read_only=True)
    s_name = serializers.CharField(source='session.session.name',read_only=True)
    #s_is_package = serializers.BooleanField(source='session.is_package',read_only=True)
    hs_name = serializers.CharField(source='hour_slot.name',read_only=True)
    dr_name = serializers.CharField(source='doctor.name',read_only=True)
    entry_status_str = serializers.CharField(source='get_entry_status_display',read_only=True)
    #session_status_str = serializers.CharField(source='session_status.name',read_only=True)
    #session_status_color = serializers.CharField(source='session_status.color_code',read_only=True)
    class Meta:
        model = PatientSessionItem
        fields=["id","pt_fullname","patient_id","s_name","hs_name","s_count","entry_status","session","is_confirmed",
                "psi_order","dr_name","entry_status_str"]
        # fields=["id","pt_fullname","patient_id","s_name","s_is_package","doctor","session","s_count","is_confirmed",
        #         "psi_order","dr_name","entry_status","entry_status_str","session_status_str","session_status_color"]
    # def to_representation(self, instance):
    #     obj = super().to_representation(instance)
    #     obj["doctor"]= instance.doctor.get_full_name() if instance.doctor else ""
    #     return obj
class PatientSessionSerializer(serializers.ModelSerializer):
    session_string = serializers.SerializerMethodField()
    patient_string = serializers.SerializerMethodField()
    created_at = serializers.DateTimeField(format="%Y-%m-%d",required=False)
    created_by_name = serializers.CharField(source='created_by.name',read_only=True)
    class Meta:
        model = PatientSession
        fields="__all__"
        #extra_kwargs = {'created_at':{'required': False}}
        read_only_fields = ('session_string', 'patient_string')

    def get_session_string(self, obj):
        return obj.session.name    
    def get_patient_string(self, obj):
        return obj.patient.fullname

class PtSsnItemCustomSerializer(serializers.ModelSerializer):
    entry_datetime = serializers.DateTimeField(format="%d-%m-%Y")
    # psi_drugs = PresPsiDrugItemSerializer(many=True, read_only=True)
    # psi_tests = PresPsiTestItemSerializer(many=True, read_only=True)
    # psi_rads = PresPsiRadiologyItemSerializer(many=True, read_only=True)
    # psi_exrszs = PsiExrszItemMiniSerializer(many=True, read_only=True)

    class Meta:
        model =  PatientSessionItem
        #fields = ("id","entry_datetime","notes_current","notes_next","next_session_days","psi_drugs","psi_tests","psi_rads","psi_exrszs","created_at")
        fields = ("id","entry_datetime","doctor","reception_notes","notes_current","notes_next","psi_order")

class PtSsnItemDaySlotSerializer(serializers.ModelSerializer):
    class Meta:
        model = PatientSessionItem
        fields=["id","hour_slot","session_day","psi_order"]
    def to_representation(self, instance):
        obj = super().to_representation(instance)
        obj["hour_slot"]= instance.hour_slot.name
        obj["session_day"]= instance.session_day.strftime("%Y-%m-%d")
        return obj
    
class PsiAssessmentSerializer(serializers.ModelSerializer):
    created_at_dt = serializers.DateTimeField(source="created_at",format="%Y-%m-%d %I:%M %p",read_only=True)
    created_by_name = serializers.CharField(source='created_by.name',read_only=True)
    doctor_name = serializers.CharField(source='doctor.name',read_only=True)
    pt_name = serializers.CharField(source='patient.fullname',read_only=True)
    assessment_code_string = serializers.CharField(source='get_assessment_code_display',read_only=True)
    class Meta:
        model = PsiAssessment
        fields="__all__"
# ############## END MedicalService%Module #################

# ############## Patient%Module #################

# class PatientSerializer(serializers.ModelSerializer):
#     #last_visit = serializers.DateTimeField(format="%d/%m/%Y",required=False)
#     last_visit = serializers.DateTimeField(format="%Y-%m-%d",required=False)
#     #location_str = serializers.SerializerMethodField()
#     #found_way_str = serializers.SerializerMethodField()
#     #next_consult = serializers.DateField(format="%d-%m-%Y",required=False)
#     #new_code_value = serializers.SerializerMethodField()
#     # company_string = serializers.SerializerMethodField()
#     class Meta:
#         model = Patient
#         fields="__all__"
#         #read_only_fields = ('new_code_value')
#     def to_representation(self, instance):
#         obj = super().to_representation(instance)
#         # obj['last_visit'] = instance.last_visit.strftime("%Y-%m-%d    %H:%M")
#         #if obj['last_visit'] : obj['last_visit'] = instance.last_visit.strftime("%Y-%m-%d")
#         #else : obj['last_visit'] = "لا توجد زيارات سابقة"
#         if obj['next_consult'] : obj['next_consult'] = instance.next_consult.strftime("%Y-%m-%d")
#         else : obj['next_consult'] = "غير محدد"
#         return obj
#     # def get_location_str(self, obj):
#     #     if obj.location :  return obj.location.name
#     #     else : return ""
#     # def get_found_way_str(self, obj):
#     #     if obj.found_way :  return obj.found_way.name
#     #     else : return ""
#     # def get_new_code_value(self, obj):
#     #     max_code = Patient.objects.aggregate(Max('code'))
#     #     if max_code["code__max"] : new_code_value = int(max_code["code__max"])+1
#     #     else : new_code_value = 1
#     #     return new_code_value
#     # def get_org_string(self, obj):
#     #     if obj.organization :  return obj.organization.name
#     #     else : return ""


# # class CommonDiseaseSerializer(serializers.ModelSerializer):
# #     class Meta:
# #         model = CommonDisease
# #         fields="__all__"

# ############## END Patient%Module #################

# ############## PatientVisit%Module #################

# class PatientVisitSerializer(serializers.ModelSerializer):
#     visit_status_string = serializers.SerializerMethodField()
#     patient_string = serializers.SerializerMethodField()
#     medicalservice_string = serializers.SerializerMethodField()
#     medicalservice_value = serializers.SerializerMethodField()
#     entry_datetime = serializers.DateTimeField(format="%d/%m/%Y %H:%M")
#     created_at = serializers.DateTimeField(format="%d-%m-%Y %H:%M")

#     class Meta:
#         model = PatientVisit
#         fields="__all__"
#         read_only_fields = ('visit_status_string', 'patient_string',"medicalservice_string","medicalservice_value")

#     def get_visit_status_string(self, obj):
#         return obj.get_visit_status_display()        
#     def get_patient_string(self, obj):
#         return obj.patient.fullname
#     def get_medicalservice_string(self, obj):
#         return obj.medicalservice.name
#     def get_medicalservice_value(self, obj):
#         return obj.medicalservice.price
# class PatientDetailVisitSerializer(serializers.ModelSerializer):
#     def create(self, validated_data):
#         amount_paid = int(validated_data['amount_paid'])
#         amount_required = int(validated_data['amount_required'])
#         amount_remaining = amount_required - amount_paid
#         patient_=validated_data['patient']
#         lastVisitObjToday = PatientVisit.objects.filter(visit_day=validated_data["visit_day"]).aggregate(Max('entry_order'))
#         entry_order = (int(lastVisitObjToday["entry_order__max"]))+1 if lastVisitObjToday["entry_order__max"] else 1
#         visitObj = PatientVisit(
#         patient=patient_,
#         medicalservice=validated_data['medicalservice'],
#         amount_paid=amount_paid,
#         amount_required=amount_required,
#         amount_remaining = amount_remaining,
#         visit_day = validated_data["visit_day"],
#         visit_status = validated_data["visit_status"],
#         entry_datetime = validated_data["entry_datetime"],
#         doctor_discount=validated_data["doctor_discount"],
#         entry_order = entry_order,
#         created_by = validated_data["created_by"])
#         patient_.last_visit = timezone.now()
#         # if amount_remaining > 0 :
#         #     patient_balance  = patient_.debt
#         #     patient_.debt = patient_balance + amount_remaining
#         # if amount_paid > amount_required : 
#         #     patient_balance  = patient_.debt
#         #     patient_.debt = patient_balance -  (amount_paid - amount_required)
#         patient_.save()
#         visitObj.save()
#         return visitObj
        
#     class Meta:
#         model = PatientVisit
#         fields="__all__"
# class PatientVisitMiniSerializer(serializers.ModelSerializer):
#     medicalservice_string = serializers.SerializerMethodField()
#     class Meta:
#         model = PatientVisit
#         fields="__all__"
#         read_only_fields = ("id",'visit_day', 'doctor_notes',"medicalservice_string","doctor","weight","bmi","diet_type")

#     def get_medicalservice_string(self, obj):
#         return obj.medicalservice.name
#     def to_representation(self, instance):
#         obj = super().to_representation(instance)
#         obj['doctor'] = instance.doctor.get_full_name() if instance.doctor else ""
#         obj['diet_type'] = instance.diet_type.name if  instance.diet_type else ""
#         return obj

# class PatientSessionItemMiniSerializer(serializers.ModelSerializer):
#     pt_fullname = serializers.CharField(source='session.patient.fullname',read_only=True)
#     patient_id = serializers.IntegerField(source='session.patient.id',read_only=True)
#     s_name = serializers.CharField(source='session.session.name',read_only=True)
#     s_count = serializers.IntegerField(source='session.sessions_count',read_only=True)
#     s_name = serializers.CharField(source='session.session.name',read_only=True)
#     class Meta:
#         model = PatientSessionItem
#         fields=["id","pt_fullname","pt_fullname","patient_id","s_name","doctor","session","s_count","psi_order"]
#     def to_representation(self, instance):
#         obj = super().to_representation(instance)
#         obj["doctor"]= instance.doctor.get_full_name() if instance.doctor else ""
#         return obj
# '''
# class CanceledVisitSerializer(serializers.ModelSerializer):
#     class Meta:
#         model = CanceledVisit
#         fields="__all__"
# class CanceledVisitCustomSerializer(serializers.ModelSerializer):
#     creation_day= serializers.SerializerMethodField()
#     creation_time= serializers.SerializerMethodField()
#     class Meta:
#         model = CanceledVisit
#         fields=("cancel_reason","patient","created_by","creation_day","creation_time",)
#     def get_creation_day(self, obj):
#         return obj.created_at.strftime("%Y-%m-%d")
#     def get_creation_time(self, obj):
#         return obj.created_at.strftime("%H:%M")
#     def to_representation(self, instance):
#         obj = super().to_representation(instance)
#         obj['patient'] = instance.patient.fullname
#         obj['created_by'] = instance.created_by.get_full_name()
#         return obj
# '''
# ############## END PatientVisit%Module #################

# ############## Prescription%Module #################

# class DrugSerializer(serializers.ModelSerializer):
#     class Meta:
#         model = Drug
#         fields="__all__"
# class DrugMiniSerializer(serializers.ModelSerializer):
#     class Meta:
#         model = Drug
#         fields=("name_en","name_ar","id")
# class RadiologySerializer(serializers.ModelSerializer):
#     class Meta:
#         model = Radiology
#         fields="__all__"
# class MedicalTestSerializer(serializers.ModelSerializer):
#     class Meta:
#         model = MedicalTest
#         fields="__all__"
# class TimeTodoSerializer(serializers.ModelSerializer):
#     class Meta:
#         model = TimeTodo
#         fields="__all__"
# class PresDrugItemSerializer(serializers.ModelSerializer):
#     item_type_string = serializers.SerializerMethodField()
#     item_type_model = serializers.SerializerMethodField()
#     item_name_string = serializers.SerializerMethodField()
#     todo_string = serializers.SerializerMethodField()
    
#     class Meta:
#         model = PresDrugItem
#         fields="__all__"
#         read_only_fields = ("item_type_string","item_type_model",'item_name_string', 'todo_string')

#     def get_item_name_string(self, obj):
#         return obj.drug.name_en
#     def get_todo_string(self, obj):
#         return obj.time_todo.name
#     def get_item_type_string(self, obj):
#         return "دواء"
#     def get_item_type_model(self, obj):
#         return "Drug"
# class PresRadiologyItemSerializer(serializers.ModelSerializer):
#     item_type_string = serializers.SerializerMethodField()
#     item_type_model = serializers.SerializerMethodField()
#     item_name_string = serializers.SerializerMethodField()
#     todo_string = serializers.SerializerMethodField()

#     class Meta:
#         model = PresRadiologyItem
#         fields="__all__"
#         read_only_fields = ("item_type_string","item_type_model",'item_name_string', 'todo_string')

#     def get_item_name_string(self, obj):
#         return "{}".format(obj.radiology.name)
#         #return "{} {} ".format(obj.radiology.name,obj.radiology.descrption)
#     def get_todo_string(self, obj):
#         return obj.time_todo.name
#     def get_item_type_string(self,obj):
#         return "أشعة"
#     def get_item_type_model(self,obj):
#         return "Radiology"
# class PresTestItemSerializer(serializers.ModelSerializer):
#     item_type_string = serializers.SerializerMethodField()
#     item_type_model = serializers.SerializerMethodField()
#     item_name_string = serializers.SerializerMethodField()
#     todo_string = serializers.SerializerMethodField()

#     class Meta:
#         model = PresTestItem
#         fields="__all__"
#         read_only_fields = ("item_type_string","item_type_model",'item_name_string', 'todo_string')
    
#     def get_item_name_string(self, obj):
#         return "{}".format(obj.medical_test.name)
#         #return "{} {} ".format(obj.medical_test.name,obj.medical_test.descrption)
#     def get_todo_string(self, obj):
#         return obj.time_todo.name
#     def get_item_type_string(self,obj):
#         return  "تحليل"
#     def get_item_type_model(self,obj):
#         return "Test"

# ############## END Prescription%Module ################


# ##############  PhysicalSessions%Module ################# 
# class PtEvaluationCardSerializer(serializers.ModelSerializer):
#     class Meta:
#         model = PtEvaluationCard
#         fields="__all__"

# class PtEvaluationCardUpdateSerializer(serializers.ModelSerializer):
#     patient = serializers.SerializerMethodField(required=False)
#     class Meta:
#         model = PtEvaluationCard
#         fields="__all__"

# class PatientSessionMiniSerializer(serializers.ModelSerializer):
#     session_string = serializers.SerializerMethodField()
#     created_at = serializers.DateTimeField(format="%Y-%m-%d")
#     class Meta:
#         model = PatientSession
#         fields="__all__"
#         read_only_fields = ("id","session_string","created_at","sessions_count","sessions_done","sessions_remain")

#     def get_session_string(self, obj):
#         return obj.session.name
    
# class PtSsnItemCustomSerializer(serializers.ModelSerializer):
#     entry_datetime = serializers.DateTimeField(format="%d-%m-%Y")
#     # psi_drugs = PresPsiDrugItemSerializer(many=True, read_only=True)
#     # psi_tests = PresPsiTestItemSerializer(many=True, read_only=True)
#     # psi_rads = PresPsiRadiologyItemSerializer(many=True, read_only=True)
#     # psi_exrszs = PsiExrszItemMiniSerializer(many=True, read_only=True)

#     class Meta:
#         model =  PatientSessionItem
#         #fields = ("id","entry_datetime","notes_current","notes_next","next_session_days","psi_drugs","psi_tests","psi_rads","psi_exrszs","created_at")
#         fields = ("id","entry_datetime","doctor","reception_notes","notes_current","notes_next","psi_order")

# class PtSsnItemDaySlotSerializer(serializers.ModelSerializer):
#     class Meta:
#         model = PatientSessionItem
#         fields=["id","hour_slot","session_day","psi_order"]
#     def to_representation(self, instance):
#         obj = super().to_representation(instance)
#         obj["hour_slot"]= instance.hour_slot.name
#         obj["session_day"]= instance.session_day.strftime("%Y-%m-%d")
#         return obj

# ############## END PhysicalSessions%Module #################